@@ -3,6 +3,9 @@
 # Univention Python
 #  LDAP access
 #
+# Like what you see? Join us!
+# https://www.univention.com/about-us/careers/vacancies/
+#
 # Copyright 2002-2022 Univention GmbH
 #
 # https://www.univention.de/
@@ -197,7 +200,7 @@ def getMachineConnection(start_tls=2, decode_ignorelist=[], ldap_master=True, se
 
 
 def _fix_reconnect_handling(func):
-	# Bug #47926: python ldap does not reconnect on ldap.UNAVAILABLE
+	# Bug #47926: Python LDAP does not reconnect on ldap.UNAVAILABLE
 	# We need this until https://github.com/python-ldap/python-ldap/pull/267 is fixed
 	@wraps(func)
 	def _decorated(self, *args, **kwargs):
@@ -230,10 +233,10 @@ class access(object):
 	:param int start_tls: 0=no, 1=try StartTLS, 2=require StartTLS.
 	:param str ca_certfile: File name to CA certificate.
 	:param decode_ignorelist: List of LDAP attribute names which shall be handled as binary attributes.
-	:param use_ldaps bool: Connect to SSL port.
-	:param uri str: LDAP connection string.
-	:param follow_referral bool: Follow referrals and return result from other servers instead of returning the referral itself.
-	:param reconnect bool: Automatically re-establish connection to LDAP server if connection breaks.
+	:param bool use_ldaps: Connect to SSL port.
+	:param str uri: LDAP connection string.
+	:param bool follow_referral: Follow referrals and return result from other servers instead of returning the referral itself.
+	:param bool reconnect: Automatically re-establish connection to LDAP server if connection breaks.
 	"""
 
 	def __init__(self, host='localhost', port=None, base='', binddn='', bindpw='', start_tls=2, ca_certfile=None, decode_ignorelist=[], use_ldaps=False, uri=None, follow_referral=False, reconnect=True):
@@ -506,7 +509,7 @@ class access(object):
 
 		if scope == 'base+one':
 			res = self.lo.search_ext_s(base, ldap.SCOPE_BASE, filter, attr, serverctrls=serverctrls, clientctrls=None, timeout=timeout, sizelimit=sizelimit) + \
-				self.lo.search_ext_s(base, ldap.SCOPE_ONELEVEL, filter, attr, serverctrls=serverctrls, clientctrls=None, timeout=timeout, sizelimit=sizelimit)
+				self.__search(base, ldap.SCOPE_ONELEVEL, filter, attr, serverctrls=serverctrls, clientctrls=None, timeout=timeout, sizelimit=sizelimit, response=response)
 		else:
 			if scope == 'sub' or scope == 'domain':
 				ldap_scope = ldap.SCOPE_SUBTREE
@@ -514,7 +517,7 @@ class access(object):
 				ldap_scope = ldap.SCOPE_ONELEVEL
 			else:
 				ldap_scope = ldap.SCOPE_BASE
-			res = self.lo.search_ext_s(base, ldap_scope, filter, attr, serverctrls=serverctrls, clientctrls=None, timeout=timeout, sizelimit=sizelimit)
+			res = self.__search(base, ldap_scope, filter, attr, serverctrls=serverctrls, clientctrls=None, timeout=timeout, sizelimit=sizelimit, response=response)
 
 		if unique and len(res) > 1:
 			raise ldap.INAPPROPRIATE_MATCHING({'desc': 'more than one object'})
@@ -522,6 +525,15 @@ class access(object):
 			raise ldap.NO_SUCH_OBJECT({'desc': 'no object'})
 		return res
 
+	def __search(self, *args, **kwargs):
+		response = kwargs.pop('response', None)
+		if isinstance(response, dict) and kwargs.get('serverctrls'):
+			rtype, res, rmsgid, resp_ctrls = self.lo.result3(self.lo.search_ext(*args, **kwargs))
+			response['ctrls'] = resp_ctrls
+			return res
+		else:
+			return self.lo.search_ext_s(*args, **kwargs)
+
 	def searchDn(self, filter='(objectClass=*)', base='', scope='sub', unique=False, required=False, timeout=-1, sizelimit=0, serverctrls=None, response=None):
 		# type: (str, str, str, bool, bool, int, int, Optional[List[ldap.controls.LDAPControl]], Optional[Dict[str, ldap.controls.LDAPControl]]) -> List[str]
 		"""
@@ -602,10 +614,10 @@ class access(object):
 		"""
 		Merge policies into result.
 
-		:param policy_dn str: Distinguished name of the policy object.
+		:param str policy_dn: Distinguished name of the policy object.
 		:param obj_dn: Distinguished name of the LDAP object.
-		:param object_classes set: the set of object classes of the LDAP object.
-		:param result list: A mapping, into which the policy is merged.
+		:param set object_classes: the set of object classes of the LDAP object.
+		:param list result: A mapping, into which the policy is merged.
 		"""
 		pattrs = self.get(policy_dn)
 		if not pattrs:
@@ -969,7 +981,7 @@ class access(object):
 		"""
 		Follow LDAP rederral.
 
-		:param exception ldap.REFERRAL: The LDAP referral exception.
+		:param ldap.REFERRAL exception: The LDAP referral exception.
 		:returns: LDAP connection object for the referred LDAP server.
 		:rtype: ldap.ldapobject.ReconnectLDAPObject
 		"""
